import time
from azul_voos import util, dates, html_scraper
from selenium import webdriver


from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.chrome.options import Options


class Query:
    def __init__(self, origem, destino, data_ida):
        self.origem = origem
        self.destino = destino
        dates.verify_ddmmyyyy(data_ida)
        dates.verify_date_is_in_future(data_ida)
        self.data_dmy = data_ida # Data no formato DD/MM/YYYY armazenada
        self.data_ida = dates.convert_ddmmyyyy_to_mmddyyyy(data_ida)

    def __str__(self):
        return f'Origem: {self.origem}, Destino: {self.destino}, Data de ida: {self.data_dmy}'

    def __repr__(self):
        return str(self)


def read_html(url, driver):
    driver.get(url)
    wait = WebDriverWait(driver, timeout = 60, poll_frequency = 0.5)

    try:
        wait.until(html_scraper.is_page_loaded)
    except:
        print('Timeout, trying again')
        try:
            driver.refresh()
            wait.until(html_scraper.is_page_loaded)
        except:
            print('Timeout again, returning None')
            return None

    if not html_scraper.is_page_loaded_and_has_flights(driver):
        return None
    html = driver.page_source
    return html


def get_flight_data_from_card(card_html):
    ''' Given a card html, returns a dictionary with flight data.
        Returns None if the flight is sold out.'''
    flight_dict = dict()
    if html_scraper.is_flight_sold_out(card_html):
        return None
    flight_dict['Normal'], flight_dict['Mais Azul'] = html_scraper.get_prices_from_card(card_html)
    flight_dict['Partida'] = html_scraper.get_departure_time_from_card(card_html)
    flight_dict['Chegada'] = html_scraper.get_arrival_time_from_card(card_html)
    flight_dict['Conexões'] = html_scraper.get_num_conexoes(card_html)
    flight_dict['Duração'] = html_scraper.get_flight_duration_from_card(card_html)
    return flight_dict


def get_data_from_single_query(query, driver, verbose=False):
    url = util.query_to_url(query)

    if verbose:
        time_start = time.time()
        print(f'Query: {query.origem} -> {query.destino} ({query.data_ida})')
        print(f'URL: {url}')
    
    html = read_html(url, driver)
    if html is None:
        return((query, []))

    flight_card_list = html_scraper.get_flight_card_list(html)
    flights_from_query = []
    for card in flight_card_list:
        if card is not None:
            flights_from_query.append(get_flight_data_from_card(card))

    if verbose:
        print(f'Found {len(flights_from_query)} flights')
        print('List of flights:')
        for flight in flights_from_query:
            print(flight)
        time_end = time.time()
        print(f'Tempo de execução: {(time_end - time_start):.2f} segundos')
        print()
    
    return flights_from_query


def get_flight_data(query_list, driver, verbose=False):
    '''
    Given a list of queries in the format (origem, destino, MM/DD/YYYY),
    returns a list of tuples with queries and flight data.
    TODO: sleep_time is the minimum time to wait between requests.
    '''
    scraped_data = []
    for query in query_list:
        scraped_data.append(get_data_from_single_query(query, driver, verbose))
    return scraped_data


class FlightScraper:
    def __init__(self, options=None, verbose=False):
        self.verbose = verbose
        self.options = options
        
        if self.options is None:
            self.options = Options()
            self.options.page_load_strategy = 'eager' # makes driver.get() faster
        try:
            self.driver = webdriver.Chrome(options=self.options)
        except:
            print('Chrome driver not found. Downloading...')
            util.download_chrome_driver()
            self.driver = webdriver.Chrome(options=self.options)
        self.driver.set_window_size(1280, 720) # If window size is too small, the wrong page will load
        self.driver.minimize_window()

    def scrape(self, query_list):
        '''
        Given a list of queries in the format (origem, destino, MM/DD/YYYY),
        returns a list of tuples with queries and flight data.
        '''
        return get_flight_data(query_list, self.driver, verbose=self.verbose)

    def close(self):
        self.driver.quit()

def main():
    # Query list example:
    origem = 'VCP'
    destino = 'FLL'
    dates = ['02/02/2023', '03/02/2023', '04/02/2023']
    queries = [Query(origem, destino, date) for date in dates]
    myScraper = FlightScraper(verbose=True)
    result = myScraper.scrape(queries)

if __name__ == '__main__':
    main()




url_template = "https://www.voeazul.com.br/br/pt/home/selecao-voo?c[0].ds=ORIGEM&c[0].std=MM/DD/YYYY&c[0].as=DESTINO&p[0].t=ADT&p[0].c=1&p[0].cp=false&f.dl=3&f.dr=3&cc=BRL"

def query_to_url(query):
    return url_template.replace('ORIGEM', query.origem).replace('DESTINO', query.destino).replace('MM/DD/YYYY', query.data_ida)

def parse_flight_duration(duration_str):
    '''Retorna a duração do voo em minutos'''
    '''duration_str é uma string no formato 'Xd Yh Zm' '''
    duration_list = duration_str.split()
    duration = 0
    for i in range(len(duration_list)):
        if duration_list[i].endswith('d'):
            duration += int(duration_list[i][:-1]) * 24 * 60
        elif duration_list[i].endswith('h'):
            duration += int(duration_list[i][:-1]) * 60
        elif duration_list[i].endswith('m'):
            duration += int(duration_list[i][:-1])
    return duration

def download_chrome_driver():
    '''Download the latest chrome driver and extract it to the current folder'''
    import requests
    import wget
    import zipfile
    import os

    # get the latest chrome driver version number
    url = 'https://chromedriver.storage.googleapis.com/LATEST_RELEASE'
    response = requests.get(url)
    version_number = response.text

    # build the donwload url
    download_url = "https://chromedriver.storage.googleapis.com/" + version_number +"/chromedriver_win32.zip"

    # download the zip file using the url built above
    latest_driver_zip = wget.download(download_url,'chromedriver.zip')

    # extract the zip file
    with zipfile.ZipFile(latest_driver_zip, 'r') as zip_ref:
        zip_ref.extractall() # you can specify the destination folder path here
    # delete the zip file downloaded above
    os.remove(latest_driver_zip)


from bs4 import BeautifulSoup
import re
from azul_voos import util

def is_page_loaded_and_has_no_flights(driver):
    html = driver.page_source
    return html.find("Parece que não temos voos") != -1

def is_page_loaded_and_has_flights(driver):
    '''"Ver tarifas" só aparece quando voos são mostrados'''
    html = driver.page_source
    return html.find("Ver tarifas") != -1

def is_page_loaded(driver):
    return is_page_loaded_and_has_flights(driver) or is_page_loaded_and_has_no_flights(driver)

def get_flight_card_list(html):
     # get all divs in which the class name starts with "flight-card", uses regex
    soup = BeautifulSoup(html, 'html.parser')
    flight_card_list = soup.find_all('div', class_=re.compile('^flight-card\s'))
    return flight_card_list

def is_flight_sold_out(card_html):
    soup = BeautifulSoup(str(card_html), 'html.parser')
    sold_out_html = str(card_html).find("Voo esgotado")
    if sold_out_html != -1:
        return True

def read_price_from_html(price_html):
    # get only the digits from the string
    price_digits = re.findall(r'\d+', price_html)
    price = int(''.join(price_digits))/100
    return price

def get_prices_from_card(card_html):
    soup = BeautifulSoup(str(card_html), 'html.parser')
    price_html_list = soup.find_all('h4', class_=re.compile('^current\s'))
    price_html_list = [e.text for e in price_html_list]
    flight_prices = [read_price_from_html(str(price_html)) for price_html in price_html_list]
    # The next error can be thrown if the page window is too small
    if(len(flight_prices) < 3):
        print(flight_prices)
    assert len(flight_prices) >= 3 #test
    # flight_prices[0] e flight_prices[1] são ambos a tarifa normal
    # Portanto flight_prices[1] não é usada
    return flight_prices[0], flight_prices[2]

def get_departure_time_from_card(card_html):
    soup = BeautifulSoup(str(card_html), 'html.parser')
    departure_time_html = soup.find_all('h4', class_=re.compile('^departure\s'))
    departure_time = re.findall(r'\d+:\d+', str(departure_time_html))
    return departure_time[0]

def get_arrival_time_from_card(card_html):
    soup = BeautifulSoup(str(card_html), 'html.parser')
    arrival_time_html = soup.find_all('h4', class_=re.compile('^arrival\s'))
    arrival_time = re.findall(r'\d+:\d+', str(arrival_time_html))
    return arrival_time[0]

def get_flight_duration_from_card(card_html):
    ''' Given a card html, returns the flight duration in minutes '''
    soup = BeautifulSoup(str(card_html), 'html.parser')
    duration_html = soup.find_all('button', class_=re.compile('^duration\s'))
    duration_strong_html = duration_html[0].find_all('strong')[0]
    duration_str = str(duration_strong_html).replace('<strong>', '').replace('</strong>', '')
    return util.parse_flight_duration(duration_str)

def get_cod_voos(card_html):
    soup = BeautifulSoup(str(card_html), 'html.parser')
    cod_voos = re.findall(r'mero\sdo\svoo\s(\d+)', str(soup))
    return cod_voos

def get_num_conexoes(card_html):
    return len(get_cod_voos(card_html)) - 1

    


import datetime

def verify_ddmmyyyy(date_text):
    '''Verifies if a date in the format DD/MM/YYYY is valid'''
    try:
        datetime.datetime.strptime(date_text, '%d/%m/%Y')
    except ValueError:
        raise ValueError(f"Incorrect data format, {date_text}. Should be DD/MM/YYYY")

def verify_date_is_in_future(date_text):
    '''Verifies if a date in the format DD/MM/YYYY is in the future'''
    date = datetime.datetime.strptime(date_text, '%d/%m/%Y')
    if date < datetime.datetime.now():
        print(f"Date {date_text} is in the past")

def convert_ddmmyyyy_to_mmddyyyy(date_text):
    '''Converts a date in the format DD/MM/YYYY to MM/DD/YYYY'''
    date = datetime.datetime.strptime(date_text, '%d/%m/%Y')
    return date.strftime('%m/%d/%Y')